/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2012 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */

/* xtt_xcolwind.cpp -- Display object attributes */

#include "glow_std.h"

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "co_cdh.h"
#include "co_dcli.h"
#include "co_time.h"
#include "rt_xnav_msg.h"

#include "flow.h"
#include "flow_browctx.h"
#include "flow_browapi.h"
#include "co_lng.h"
#include "xtt_xcolwind_gtk.h"
#include "xtt_xattnav_gtk.h"
#include "xtt_xnav.h"
#include "cow_wow_gtk.h"
#include "rt_xatt_msg.h"

CoWowRecall XColWindGtk::value_recall;

void XColWindGtk::message( char severity, const char *message)
{
  gtk_label_set_text( GTK_LABEL(msg_label), message);
}

void XColWindGtk::set_prompt( const char *prompt)
{
  if ( strcmp(prompt, "") == 0) {
    g_object_set( cmd_prompt, "visible", FALSE, NULL);
    g_object_set( msg_label, "visible", TRUE, NULL);
  }
  else {
    g_object_set( msg_label, "visible", FALSE, NULL);
    g_object_set( cmd_prompt, "visible", TRUE, NULL);
  }
}

void XColWindGtk::change_value( int set_focus)
{
  int		sts;
  GtkWidget	*text_w;
  int		multiline;
  char		*value;
  int		input_size;

  if ( input_open) {
    g_object_set( cmd_input, "visible", FALSE, NULL);
    g_object_set( cmd_scrolledinput, "visible", FALSE, NULL);
    set_prompt( "");
    input_open = 0;
    return;
  }

  sts = xattnav->check_attr( &multiline, &input_node, input_name,
		&value, &input_size);
  if ( EVEN(sts)) {
    if ( sts == XATT__NOATTRSEL)
      message( 'E', "No attribute is selected");
    else
      message( 'E', XNav::get_message( sts));
    return;
  }

  if ( multiline) {
    text_w = cmd_scrolledinput;
    g_object_set( text_w, "visible", TRUE, NULL);

    int w, h;
    gdk_drawable_get_size( pane->window, &w, &h);
    gtk_paned_set_position( GTK_PANED(pane), h - 170);
    if ( set_focus)
      gtk_widget_grab_focus( cmd_scrolledtextview);
    input_max_length = input_size - 1;
    input_multiline = 1;
  }
  else {
    text_w = cmd_input;
    g_object_set( text_w, "visible", TRUE,
		  "max-length", input_size - 1, NULL);
    if ( set_focus)
      gtk_widget_grab_focus( cmd_input);
    input_multiline = 0;
  }

  message( ' ', "");

  if ( value) {
    if ( multiline) {
      GtkTextIter start_iter, end_iter;
      gtk_text_buffer_get_start_iter( cmd_scrolled_buffer, &start_iter);
      gtk_text_buffer_get_end_iter( cmd_scrolled_buffer, &end_iter);
      gtk_text_buffer_delete( cmd_scrolled_buffer, &start_iter, &end_iter);

      gtk_text_buffer_get_start_iter( cmd_scrolled_buffer, &start_iter);
      gtk_text_buffer_insert( cmd_scrolled_buffer, &start_iter, value, -1);

      // Select the text
      // gtk_text_buffer_get_start_iter( cmd_scrolled_buffer, &start_iter);
      // gtk_text_buffer_get_end_iter( cmd_scrolled_buffer, &end_iter);
      // gtk_text_buffer_select_range( cmd_scrolled_buffer, &start_iter, &end_iter);
    }
    else {
      gint pos = 0;
      gtk_editable_delete_text( GTK_EDITABLE(cmd_input), 0, -1);
      gtk_editable_insert_text( GTK_EDITABLE(text_w), value, strlen(value), &pos);

      // Select the text
      gtk_editable_set_position( GTK_EDITABLE(cmd_input), -1);
      gtk_editable_select_region( GTK_EDITABLE(cmd_input), 0, -1);
    }
  }
  else {
    gtk_editable_delete_text( GTK_EDITABLE(cmd_input), 0, -1);
  }

  message( ' ', "");
  set_prompt( Lng::translate("value >"));
  input_open = 1;
}

void XColWindGtk::action_text_inserted( GtkTextBuffer *w, GtkTextIter *iter, gchar *str, gint len, gpointer data)
{
  XColWindGtk *xcolwind = (XColWindGtk *)data;

  int count = gtk_text_buffer_get_char_count( w);  

  if ( count > xcolwind->input_max_length) {
    // Remove inserted chars (note that iter now points at the end of the inserted text)
    GtkTextIter start_iter;

    int offs = gtk_text_iter_get_offset( iter);
    gtk_text_buffer_get_iter_at_offset( w, &start_iter, offs - len);
    gtk_text_buffer_delete( w, &start_iter, iter);

    CoWowGtk wow( xcolwind->toplevel);
    wow.DisplayError( "Error message", "Attribute size exceeded");
  }
}

//
//  Callbackfunctions from menu entries
//
void XColWindGtk::activate_change_value( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->change_value(1);
}

void XColWindGtk::activate_open( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_open();
}

void XColWindGtk::activate_save( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_save();
}

void XColWindGtk::activate_saveas( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_saveas();
}

void XColWindGtk::activate_print( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_print();
}

void XColWindGtk::activate_close_changeval( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->change_value_close();
}

void XColWindGtk::activate_exit( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  if ( xcolwind->close_cb)
    (xcolwind->close_cb)( xcolwind->parent_ctx, (void *)xcolwind);
  else
    delete xcolwind;
}

void XColWindGtk::activate_insert( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_collect_insert();
}

void XColWindGtk::activate_delete( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_collect_delete();
}

void XColWindGtk::activate_moveup( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_moveup();
}

void XColWindGtk::activate_movedown( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_movedown();
}

void XColWindGtk::activate_display_object( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_display_object();
}

void XColWindGtk::activate_show_cross( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_show_cross();
}

void XColWindGtk::activate_open_classgraph( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_open_classgraph();
}

void XColWindGtk::activate_open_plc( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_open_plc();
}

void XColWindGtk::activate_zoomin(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_zoomin();
}

void XColWindGtk::activate_zoomout(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_zoomout();
}

void XColWindGtk::activate_zoomreset(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->xattnav->unzoom();
}

void XColWindGtk::activate_scantime1(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->xattnav->set_scantime( 500);
}

void XColWindGtk::activate_scantime2(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->xattnav->set_scantime( 200);
}

void XColWindGtk::activate_scantime3(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->xattnav->set_scantime( 100);
}

void XColWindGtk::activate_scantime4(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->xattnav->set_scantime( 50);
}

void XColWindGtk::activate_scantime5(GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->xattnav->set_scantime( 20);
}

void XColWindGtk::activate_help( GtkWidget *w, gpointer data)
{
  XColWind *xcolwind = (XColWind *)data;

  xcolwind->activate_help();
}

gboolean XColWindGtk::action_inputfocus( GtkWidget *w, GdkEvent *event, gpointer data)
{
  XColWindGtk *xcolwind = (XColWindGtk *)data;
  gboolean scrolledinput_visible;
  gboolean input_visible;

  if ( xcolwind->focustimer.disabled())
    return TRUE;

  g_object_get( xcolwind->cmd_scrolledinput, "visible", &scrolledinput_visible, NULL);
  g_object_get( xcolwind->cmd_input, "visible", &input_visible, NULL);
  if ( scrolledinput_visible)
    gtk_widget_grab_focus( xcolwind->cmd_scrolledtextview);
  else if ( input_visible)
    gtk_widget_grab_focus( xcolwind->cmd_input);
  else if ( xcolwind->xattnav)
    xcolwind->xattnav->set_inputfocus();

  xcolwind->focustimer.disable( 400);

  return FALSE;
}

#if 0
void XColWindGtk::valchanged_cmd_input( Widget w, XEvent *event)
{
  XColWind 	*xcolwind;
  int 	sts;
  char 	*text;
  Arg 	args[2];

  XtSetArg(args[0], XmNuserData, &xcolwind);
  XtGetValues(w, args, 1);

  sts = mrm_TextInput( w, event, (char *)XColWind::value_recall, sizeof(XColWind::value_recall[0]),
	sizeof( XColWind::value_recall)/sizeof(XColWind::value_recall[0]),
	&xcolwind->value_current_recall);
  if ( sts)
  {
    text = XmTextGetString( w);
    if ( xcolwind->input_open)
    {
      sts = xcolwind->xattnav->set_attr_value( xcolwind->input_node, 
		xcolwind->input_name, text);
      XtUnmanageChild( w);
      xcolwind->set_prompt( "");
      xcolwind->input_open = 0;
      if ( xcolwind->redraw_cb)
        (xcolwind->redraw_cb)( xcolwind);

      xcolwind->xattnav->set_inputfocus();
    }
  }
}
#endif

void XColWindGtk::change_value_close()
{
  int sts;

  if ( input_open) {
    if ( input_multiline) {
      gchar *text, *textutf8;
      GtkTextIter start_iter, end_iter;
      gtk_text_buffer_get_start_iter( cmd_scrolled_buffer, &start_iter);
      gtk_text_buffer_get_end_iter( cmd_scrolled_buffer, &end_iter);

      textutf8 = gtk_text_buffer_get_text( cmd_scrolled_buffer, &start_iter, &end_iter,
				       FALSE);
      text = g_convert( textutf8, -1, "ISO8859-1", "UTF-8", NULL, NULL, NULL);
      g_free( textutf8);

      sts = xattnav->set_attr_value( input_node,
				     input_name, text);
      g_free( text);
      g_object_set( cmd_scrolledinput, "visible", FALSE, NULL);
      set_prompt( "");
      input_open = 0;

      int w, h;
      gdk_drawable_get_size( pane->window, &w, &h);
      gtk_paned_set_position( GTK_PANED(pane), h - 50);

      xattnav->redraw();
      xattnav->set_inputfocus();
    }
    else {
      char *text, *textutf8;

      textutf8 = gtk_editable_get_chars( GTK_EDITABLE(cmd_input), 0, -1);
      text = g_convert( textutf8, -1, "ISO8859-1", "UTF-8", NULL, NULL, NULL);
      g_free( textutf8);

      sts = xattnav->set_attr_value( input_node, 
				     input_name, text);
      g_free( text);
      g_object_set( cmd_input, "visible", FALSE, NULL);
      set_prompt( "");
      input_open = 0;
      if ( redraw_cb)
        (redraw_cb)( this);

      xattnav->set_inputfocus();
    }
  }
}

void XColWindGtk::activate_cmd_input( GtkWidget *w, gpointer data)
{
  char *text, *textutf8;
  XColWindGtk *xcolwind = (XColWindGtk *)data;
  int sts;

  g_object_set( xcolwind->cmd_prompt, "visible", FALSE, NULL);
  g_object_set( xcolwind->cmd_input, "visible", FALSE, NULL);

  xcolwind->xattnav->set_inputfocus();

  textutf8 = gtk_editable_get_chars( GTK_EDITABLE(w), 0, -1);
  text = g_convert( textutf8, -1, "ISO8859-1", "UTF-8", NULL, NULL, NULL);
  g_free( textutf8);

  if ( xcolwind->input_open) {
    sts = xcolwind->xattnav->set_attr_value( xcolwind->input_node, 
					 xcolwind->input_name, text);
    g_object_set( w, "visible", FALSE, NULL);
    xcolwind->set_prompt( "");
    xcolwind->input_open = 0;
    if ( xcolwind->redraw_cb)
      (xcolwind->redraw_cb)( xcolwind);
  }
  g_free( text);
}

void XColWindGtk::activate_cmd_scrolled_ok( GtkWidget *w, gpointer data)
{
  XColWindGtk *xcolwind = (XColWindGtk *)data;
  gchar *text, *textutf8;
  int sts;

  if ( xcolwind->input_open) {
    GtkTextIter start_iter, end_iter;
    gtk_text_buffer_get_start_iter( xcolwind->cmd_scrolled_buffer, &start_iter);
    gtk_text_buffer_get_end_iter( xcolwind->cmd_scrolled_buffer, &end_iter);

    textutf8 = gtk_text_buffer_get_text( xcolwind->cmd_scrolled_buffer, &start_iter, &end_iter,
				     FALSE);
    text = g_convert( textutf8, -1, "ISO8859-1", "UTF-8", NULL, NULL, NULL);
    g_free( textutf8);

    sts = xcolwind->xattnav->set_attr_value( xcolwind->input_node,
						      xcolwind->input_name, text);
    g_object_set( xcolwind->cmd_scrolledinput, "visible", FALSE, NULL);
    xcolwind->set_prompt( "");
    xcolwind->input_open = 0;

    int w, h;
    gdk_drawable_get_size( xcolwind->pane->window, &w, &h);
    gtk_paned_set_position( GTK_PANED(xcolwind->pane), h - 50);

    xcolwind->xattnav->redraw();
    xcolwind->xattnav->set_inputfocus();
    g_free( text);
  }
}

void XColWindGtk::activate_cmd_scrolled_ca( GtkWidget *w, gpointer data)
{
  XColWindGtk *xcolwind = (XColWindGtk *)data;

  if ( xcolwind->input_open) {
    g_object_set( xcolwind->cmd_scrolledinput, "visible", FALSE, NULL);

    int w, h;
    gdk_drawable_get_size( xcolwind->pane->window, &w, &h);
    gtk_paned_set_position( GTK_PANED(xcolwind->pane), h - 50);

    xcolwind->set_prompt( "");
    xcolwind->input_open = 0;
    xcolwind->xattnav->set_inputfocus();
  }
}

void XColWindGtk::pop()
{
  gtk_window_present( GTK_WINDOW(toplevel));
}

void XColWindGtk::set_title( char *t)
{
  strncpy( title, t, sizeof(title));
  gtk_window_set_title( GTK_WINDOW(toplevel), title);
}

void XColWindGtk::set_window_size( int w, int h)
{
  gtk_window_resize( GTK_WINDOW(toplevel), w, h);
}

void XColWindGtk::get_window_size( int *w, int *h)
{
  gtk_window_get_size( GTK_WINDOW(toplevel), w, h);
}

void XColWindGtk::print()
{
  pwr_tStatus sts;

  wow->CreateBrowPrintDialog( title, xattnav->brow->ctx, flow_eOrientation_Portrait, 1.0,
			      (void *)toplevel, &sts);
}

XColWindGtk::~XColWindGtk()
{
  delete (XAttNav *)xattnav;
  delete cmd_entry;
  gtk_widget_destroy( toplevel);
}

static gint delete_event( GtkWidget *w, GdkEvent *event, gpointer data)
{
  XColWindGtk *xcolwind = (XColWindGtk *)data;

  if ( xcolwind->close_cb)
    (xcolwind->close_cb)( xcolwind->parent_ctx, (void *)xcolwind);
  else
    delete xcolwind;
  
  return TRUE;
}

static void destroy_event( GtkWidget *w, gpointer data)
{
}

XColWindGtk::XColWindGtk( GtkWidget 	*xa_parent_wid,
			  void 		*xa_parent_ctx, 
			  pwr_sAttrRef 	*xa_objar_list,
			  char		*xa_title,
			  int 		xa_advanced_user,
			  xcolwind_eType xa_type,
			  int           *xa_sts) :
  XColWind( xa_parent_ctx, xa_objar_list, xa_title, xa_advanced_user, xa_type, xa_sts),
  parent_wid(xa_parent_wid)
{
  int sts;
  pwr_tAttrRef 	ar;
  xattnav_eType xattnav_type;

  toplevel = (GtkWidget *) g_object_new( GTK_TYPE_WINDOW, 
					 "default-height", 600,
					 "default-width", 420,
					 "title", CoWowGtk::convert_utf8(title),
					 NULL);

  g_signal_connect( toplevel, "delete_event", G_CALLBACK(delete_event), this);
  g_signal_connect( toplevel, "destroy", G_CALLBACK(destroy_event), this);
  g_signal_connect( toplevel, "focus-in-event", G_CALLBACK(action_inputfocus), this);

  CoWowGtk::SetWindowIcon( toplevel);

  GtkWidget *vbox = gtk_vbox_new( FALSE, 0);

  // Menu
  // Accelerators
  GtkAccelGroup *accel_g = (GtkAccelGroup *) g_object_new(GTK_TYPE_ACCEL_GROUP, NULL);
  gtk_window_add_accel_group(GTK_WINDOW(toplevel), accel_g);

  GtkMenuBar *menu_bar = (GtkMenuBar *) g_object_new(GTK_TYPE_MENU_BAR, NULL);

  // File entry
  GtkWidget *file_close = gtk_image_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_Close"));
  gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(file_close), 
				 gtk_image_new_from_stock( "gtk-close", GTK_ICON_SIZE_MENU));
  g_signal_connect(file_close, "activate", G_CALLBACK(activate_exit), this);
  gtk_widget_add_accelerator( file_close, "activate", accel_g,
			      'w', GdkModifierType(GDK_CONTROL_MASK), GTK_ACCEL_VISIBLE);

  GtkWidget *file_save = gtk_image_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_Save"));
  gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(file_save), 
				 gtk_image_new_from_stock( "gtk-save", GTK_ICON_SIZE_MENU));
  g_signal_connect(file_save, "activate", G_CALLBACK(activate_save), this);
  gtk_widget_add_accelerator( file_save, "activate", accel_g,
			      's', GdkModifierType(GDK_CONTROL_MASK), GTK_ACCEL_VISIBLE);

  GtkWidget *file_saveas = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("S_ave as"));
  g_signal_connect( file_saveas, "activate", 
		    G_CALLBACK(activate_saveas), this);

  GtkWidget *file_open = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_Open"));
  g_signal_connect( file_open, "activate", 
		    G_CALLBACK(activate_open), this);

  GtkWidget *file_print = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_Print"));
  g_signal_connect( file_print, "activate", 
		    G_CALLBACK(activate_print), this);


  GtkMenu *file_menu = (GtkMenu *) g_object_new( GTK_TYPE_MENU, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), file_open);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), file_save);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), file_saveas);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), file_print);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), file_close);

  GtkWidget *file = gtk_menu_item_new_with_mnemonic(CoWowGtk::translate_utf8("_File"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), file);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file), GTK_WIDGET(file_menu));

  // Edit entry

  GtkWidget *edit_insert = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_Insert"));
  g_signal_connect( edit_insert, "activate", 
		    G_CALLBACK(activate_insert), this);
  gtk_widget_add_accelerator( edit_insert, "activate", accel_g,
			      'v', GdkModifierType(GDK_CONTROL_MASK), GTK_ACCEL_VISIBLE);

  GtkWidget *edit_delete = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_Delete"));
  g_signal_connect( edit_delete, "activate", 
		    G_CALLBACK(activate_delete), this);
  gtk_widget_add_accelerator( edit_delete, "activate", accel_g,
			      GDK_Delete, GdkModifierType(0), GTK_ACCEL_VISIBLE);

  GtkWidget *edit_moveup = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("Move _Up"));
  g_signal_connect( edit_moveup, "activate", 
		    G_CALLBACK(activate_moveup), this);
  gtk_widget_add_accelerator( edit_moveup, "activate", accel_g,
			      GDK_Up, GdkModifierType(GDK_SHIFT_MASK), GTK_ACCEL_VISIBLE);

  GtkWidget *edit_movedown = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("Move _Down"));
  g_signal_connect( edit_movedown, "activate", 
		    G_CALLBACK(activate_movedown), this);
  gtk_widget_add_accelerator( edit_movedown, "activate", accel_g,
			      GDK_Down, GdkModifierType(GDK_SHIFT_MASK), GTK_ACCEL_VISIBLE);


  GtkMenu *edit_menu = (GtkMenu *) g_object_new( GTK_TYPE_MENU, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), edit_insert);
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), edit_delete);
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), edit_moveup);
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), edit_movedown);

  GtkWidget *edit = gtk_menu_item_new_with_mnemonic(CoWowGtk::translate_utf8("_Edit"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), edit);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit), GTK_WIDGET(edit_menu));

  // Functions entry
  GtkWidget *func_changevalue = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("Change _Value"));
  g_signal_connect( func_changevalue, "activate", 
		    G_CALLBACK(activate_change_value), this);
  gtk_widget_add_accelerator( func_changevalue, "activate", accel_g,
			      'q', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  GtkWidget *func_close_changeval = gtk_menu_item_new_with_mnemonic( 
                      CoWowGtk::translate_utf8("C_lose Change Value"));
  g_signal_connect( func_close_changeval, "activate", 
		    G_CALLBACK(activate_close_changeval), this);
  gtk_widget_add_accelerator( func_close_changeval, "activate", accel_g,
			      't', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  GtkWidget *functions_open_plc = gtk_menu_item_new_with_mnemonic( 
				CoWowGtk::translate_utf8("Open _Program"));
  g_signal_connect( functions_open_plc, "activate", 
		    G_CALLBACK(activate_open_plc), this);
  gtk_widget_add_accelerator( functions_open_plc, "activate", accel_g,
  			      'l', GdkModifierType(GDK_CONTROL_MASK), 
  			      GTK_ACCEL_VISIBLE);

  GtkWidget *functions_display_object = gtk_menu_item_new_with_mnemonic( 
                         CoWowGtk::translate_utf8("_Display object in Navigator"));
  g_signal_connect( functions_display_object, "activate", 
		    G_CALLBACK(activate_display_object), this);
  gtk_widget_add_accelerator( functions_display_object, "activate", accel_g,
  			      'd', GdkModifierType(GDK_CONTROL_MASK), 
  			      GTK_ACCEL_VISIBLE);

  GtkWidget *functions_show_cross = gtk_menu_item_new_with_mnemonic( 
                            CoWowGtk::translate_utf8("Show C_rossreferences"));
  g_signal_connect( functions_show_cross, "activate", 
		    G_CALLBACK(activate_show_cross), this);
  gtk_widget_add_accelerator( functions_show_cross, "activate", accel_g,
  			      'r', GdkModifierType(GDK_CONTROL_MASK), 
  			      GTK_ACCEL_VISIBLE);

  GtkWidget *functions_open_classgraph = gtk_menu_item_new_with_mnemonic( 
                     CoWowGtk::translate_utf8("Open _ClassGraph"));
  g_signal_connect( functions_open_classgraph, "activate", 
		    G_CALLBACK(activate_open_classgraph), this);
  gtk_widget_add_accelerator( functions_open_classgraph, "activate", accel_g,
  			      'g', GdkModifierType(GDK_CONTROL_MASK), 
  			      GTK_ACCEL_VISIBLE);


  GtkMenu *func_menu = (GtkMenu *) g_object_new( GTK_TYPE_MENU, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(func_menu), func_changevalue);
  gtk_menu_shell_append(GTK_MENU_SHELL(func_menu), func_close_changeval);
  gtk_menu_shell_append(GTK_MENU_SHELL(func_menu), functions_open_plc);
  gtk_menu_shell_append(GTK_MENU_SHELL(func_menu), functions_display_object);
  gtk_menu_shell_append(GTK_MENU_SHELL(func_menu), functions_show_cross);
  gtk_menu_shell_append(GTK_MENU_SHELL(func_menu), functions_open_classgraph);

  GtkWidget *functions = gtk_menu_item_new_with_mnemonic(CoWowGtk::translate_utf8("_Functions"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), functions);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(functions), GTK_WIDGET(func_menu));

  // View Entry

  GtkWidget *view_zoom_in = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_IN, NULL);
  g_signal_connect( view_zoom_in, "activate", 
		    G_CALLBACK(activate_zoomin), this);
  gtk_widget_add_accelerator( view_zoom_in, "activate", accel_g,
  			      'i', GdkModifierType(GDK_CONTROL_MASK), 
  			      GTK_ACCEL_VISIBLE);

  GtkWidget *view_zoom_out = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_OUT, NULL);
  g_signal_connect( view_zoom_out, "activate", 
		    G_CALLBACK(activate_zoomout), this);
  gtk_widget_add_accelerator( view_zoom_out, "activate", accel_g,
  			      'o', GdkModifierType(GDK_CONTROL_MASK), 
  			      GTK_ACCEL_VISIBLE);

  GtkWidget *view_zoom_reset = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_100, NULL);
  g_signal_connect( view_zoom_reset, "activate", 
		    G_CALLBACK(activate_zoomreset), this);
  gtk_widget_add_accelerator( view_zoom_reset, "activate", accel_g,
  			      'b', GdkModifierType(GDK_CONTROL_MASK), 
  			      GTK_ACCEL_VISIBLE);

  // Submenu ScanTime
  GSList *view_sc_group = NULL;
  GtkWidget *view_scantime1 = gtk_radio_menu_item_new_with_mnemonic( view_sc_group, 
									"0.50 s");
  view_sc_group = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM(view_scantime1));
  g_signal_connect( view_scantime1, "activate", 
		    G_CALLBACK(activate_scantime1), this);

  GtkWidget *view_scantime2 = gtk_radio_menu_item_new_with_mnemonic( view_sc_group, 
									"0.20 s");
  view_sc_group = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM(view_scantime1));
  g_signal_connect( view_scantime2, "activate", 
		    G_CALLBACK(activate_scantime2), this);

  GtkWidget *view_scantime3 = gtk_radio_menu_item_new_with_mnemonic( view_sc_group, 
									"0.10 s");
  view_sc_group = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM(view_scantime1));
  g_signal_connect( view_scantime3, "activate", 
		    G_CALLBACK(activate_scantime3), this);

  GtkWidget *view_scantime4 = gtk_radio_menu_item_new_with_mnemonic( view_sc_group, 
									"0.05 s");
  view_sc_group = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM(view_scantime1));
  g_signal_connect( view_scantime4, "activate", 
		    G_CALLBACK(activate_scantime4), this);

  GtkWidget *view_scantime5 = gtk_radio_menu_item_new_with_mnemonic( view_sc_group, 
									"0.02 s");
  g_signal_connect( view_scantime5, "activate", 
		    G_CALLBACK(activate_scantime5), this);

  GtkWidget *view_sc = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_ScanTime"));
  GtkMenu *view_sc_menu = (GtkMenu *) g_object_new( GTK_TYPE_MENU, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_sc_menu), view_scantime1);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_sc_menu), view_scantime2);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_sc_menu), view_scantime3);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_sc_menu), view_scantime4);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_sc_menu), view_scantime5);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(view_sc),
			    GTK_WIDGET(view_sc_menu));

  GtkMenu *view_menu = (GtkMenu *) g_object_new( GTK_TYPE_MENU, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), view_zoom_in);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), view_zoom_out);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), view_zoom_reset);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), view_sc);

  GtkWidget *view = gtk_menu_item_new_with_mnemonic( CoWowGtk::translate_utf8("_View"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), view);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(view), GTK_WIDGET(view_menu));

  // Help entry
  GtkWidget *help_help = gtk_image_menu_item_new_with_mnemonic(CoWowGtk::translate_utf8("_Help"));
  gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM(help_help), 
				 gtk_image_new_from_stock( "gtk-help", GTK_ICON_SIZE_MENU));
  g_signal_connect(help_help, "activate", G_CALLBACK(activate_help), this);
  gtk_widget_add_accelerator( help_help, "activate", accel_g,
			      'h', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  GtkMenu *help_menu = (GtkMenu *) g_object_new( GTK_TYPE_MENU, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(help_menu), help_help);

  GtkWidget *help = gtk_menu_item_new_with_mnemonic(CoWowGtk::translate_utf8("_Help"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), help);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(help), GTK_WIDGET(help_menu));

  pane = gtk_vpaned_new();
  memset( &ar, 0, sizeof(ar));

  switch( type) {
  case xcolwind_eType_Collect:
    xattnav_type = xattnav_eType_Collect;
    break;
  case xcolwind_eType_CollectSignals:
    xattnav_type = xattnav_eType_CollectSignals;
    break;
  case xcolwind_eType_CollectIOSignals:
    xattnav_type = xattnav_eType_CollectSignals;
    break;
  }

  xattnav = new XAttNavGtk( (void *)this, pane, xattnav_type,
			    "Plant", &ar, xa_advanced_user, objar_list, &brow_widget, &sts);
  xattnav->message_cb = &message_cb;
  xattnav->change_value_cb = &change_value_cb;
  xattnav->popup_menu_cb = &xcolwind_popup_menu_cb;
  xattnav->is_authorized_cb = &xcolwind_is_authorized_cb;
  xattnav->init_cb = &init_cb;

  GtkWidget *statusbar = gtk_hbox_new( FALSE, 0);
  msg_label = gtk_label_new( "");
  gtk_widget_set_size_request( msg_label, -1, 25);
  cmd_prompt = gtk_label_new( "value > ");
  gtk_widget_set_size_request( cmd_prompt, -1, 25);
  cmd_entry = new CoWowEntryGtk( &value_recall);
  cmd_input = cmd_entry->widget();
  gtk_widget_set_size_request( cmd_input, -1, 25);
  g_signal_connect( cmd_input, "activate", 
		    G_CALLBACK(activate_cmd_input), this);

  gtk_box_pack_start( GTK_BOX(statusbar), msg_label, FALSE, FALSE, 0);
  gtk_box_pack_start( GTK_BOX(statusbar), cmd_prompt, FALSE, FALSE, 0);
  gtk_box_pack_start( GTK_BOX(statusbar), cmd_input, TRUE, TRUE, 0);

  gtk_box_pack_start( GTK_BOX(vbox), GTK_WIDGET(menu_bar), FALSE, FALSE, 0);
  gtk_box_pack_end( GTK_BOX(vbox), GTK_WIDGET(pane), TRUE, TRUE, 0);

  gtk_paned_pack1( GTK_PANED(pane), GTK_WIDGET(brow_widget), TRUE, TRUE);
  gtk_paned_pack2( GTK_PANED(pane), GTK_WIDGET(statusbar), FALSE, TRUE);
  
  gtk_container_add( GTK_CONTAINER(toplevel), vbox);

  cmd_scrolled_buffer = gtk_text_buffer_new( NULL);
  g_signal_connect_after( cmd_scrolled_buffer, "insert-text", 
 		    G_CALLBACK(action_text_inserted), this);

  cmd_scrolledtextview = gtk_text_view_new_with_buffer( cmd_scrolled_buffer);
  GtkWidget *viewport = gtk_viewport_new( NULL, NULL);
  GtkWidget *scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_container_add( GTK_CONTAINER(viewport), cmd_scrolledtextview);
  gtk_container_add( GTK_CONTAINER(scrolledwindow), viewport);

  cmd_scrolled_ok = gtk_button_new_with_label( "Ok");
  gtk_widget_set_size_request( cmd_scrolled_ok, 70, 25);
  g_signal_connect( cmd_scrolled_ok, "clicked", 
 		    G_CALLBACK(activate_cmd_scrolled_ok), this);
  cmd_scrolled_ca = gtk_button_new_with_label( "Cancel");
  gtk_widget_set_size_request( cmd_scrolled_ca, 70, 25);
  g_signal_connect( cmd_scrolled_ca, "clicked", 
 		    G_CALLBACK(activate_cmd_scrolled_ca), this);

  GtkWidget *hboxbuttons = gtk_hbox_new( TRUE, 40);
  gtk_box_pack_start( GTK_BOX(hboxbuttons), cmd_scrolled_ok, FALSE, FALSE, 0);
  gtk_box_pack_end( GTK_BOX(hboxbuttons), cmd_scrolled_ca, FALSE, FALSE, 0);

  cmd_scrolledinput = gtk_vbox_new( FALSE, 0);
  gtk_box_pack_start( GTK_BOX(cmd_scrolledinput), scrolledwindow, TRUE, TRUE, 0);
  gtk_box_pack_start( GTK_BOX(cmd_scrolledinput), hboxbuttons, FALSE, FALSE, 5);

  gtk_box_pack_start( GTK_BOX(statusbar), cmd_scrolledinput, TRUE, TRUE, 0);

  gtk_widget_show_all( toplevel);

  g_object_set( cmd_prompt, "visible", FALSE, NULL);
  g_object_set( cmd_input, "visible", FALSE, NULL);
  g_object_set( cmd_scrolledinput, "visible", FALSE, NULL);
  
  int w, h;
  gdk_drawable_get_size( pane->window, &w, &h);
  gtk_paned_set_position( GTK_PANED(pane), h - 50);

  wow = new CoWowGtk( toplevel);

  *xa_sts = XATT__SUCCESS;
}



