/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2012 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */

/* xtt_ev.cpp -- Alarm and event window in xtt */

#include "glow_std.h"

#include <stdio.h>
#include <stdlib.h>

#include "pwr_privilege.h"
#include "co_cdh.h"
#include "co_time.h"
#include "co_dcli.h"
#include "cow_wow.h"
#include "pwr_baseclasses.h"
#include "rt_gdh.h"
#include "rt_mh.h"
#include "rt_mh_outunit.h"
#include "rt_mh_util.h"

#include "co_lng.h"
#include "xtt_ev.h"
#include "xtt_methodtoolbar.h"
#include "rt_xnav_msg.h"

// Static pointer to Ev for mh callbacks
Ev *Ev::ev = 0;


Ev::Ev( void *ev_parent_ctx,
	char *eve_name,
	char *ala_name,
	char *blk_name,
	pwr_tObjid ev_user,
	int display_ala,
	int display_eve,
	int display_blk,
	int display_return,
	int display_ack,
	int ev_beep,
	pwr_tMask ev_pop_mask,
	int ev_eventname_seg,
	pwr_tStatus *status) :
  parent_ctx(ev_parent_ctx),
  user(ev_user), eve_display_ack(display_ack), 
  eve_display_return(display_return),
  start_trace_cb(NULL), display_in_xnav_cb(NULL), update_info_cb(NULL),
  help_cb(NULL), popup_menu_cb(0), sound_cb(0), pop_cb(0), is_authorized_cb(0), eve(NULL), ala(NULL),
  blk(0), connected(0), ala_displayed(0), eve_displayed(0), beep(ev_beep), pop_mask(ev_pop_mask), 
  eventname_seg(ev_eventname_seg), sala_cnt(0)
{
}


//
//  Delete ev
//
Ev::~Ev()
{
}

void Ev::eve_start_trace_cb( void *ctx, pwr_tObjid objid, char *name)
{
  if ( ((Ev *)ctx)->start_trace_cb)
    ((Ev *)ctx)->start_trace_cb( ((Ev *)ctx)->parent_ctx, objid, name);
}

void Ev::ala_start_trace_cb( void *ctx, pwr_tObjid objid, char *name)
{
  if ( ((Ev *)ctx)->start_trace_cb)
    ((Ev *)ctx)->start_trace_cb( ((Ev *)ctx)->parent_ctx, objid, name);
}

void Ev::blk_start_trace_cb( void *ctx, pwr_tObjid objid, char *name)
{
  if ( ((Ev *)ctx)->start_trace_cb)
    ((Ev *)ctx)->start_trace_cb( ((Ev *)ctx)->parent_ctx, objid, name);
}

void Ev::ev_popup_menu_cb( void *ctx, pwr_tAttrRef attrref,
			      unsigned long item_type, unsigned long utility,
			      char *arg, int x, int y)
{
  if ( ((Ev *)ctx)->popup_menu_cb)
    (((Ev *)ctx)->popup_menu_cb) ( ((Ev *)ctx)->parent_ctx, attrref, item_type,
				   utility, arg, x, y);
}

int Ev::ev_sound_cb( void *ctx, pwr_tAttrRef *attrref)
{
  if ( ((Ev *)ctx)->sound_cb)
    return (((Ev *)ctx)->sound_cb) ( ((Ev *)ctx)->parent_ctx, attrref);
  return 0;
}

void Ev::eve_selection_changed_cb( void *ctx)
{
  ((Ev *)ctx)->eve_methodtoolbar->set_sensitive();
  ((Ev *)ctx)->eve_sup_methodtoolbar->set_sensitive();
}

void Ev::ala_selection_changed_cb( void *ctx)
{
  ((Ev *)ctx)->ala_methodtoolbar->set_sensitive();
  ((Ev *)ctx)->ala_sup_methodtoolbar->set_sensitive();
}

char *Ev::ev_name_to_alias_cb( void *ctx, char *name)
{
  return ((Ev *)ctx)->name_to_alias( name);
}

void Ev::eve_display_in_xnav_cb( void *ctx, pwr_tAttrRef *arp)
{
  if ( ((Ev *)ctx)->display_in_xnav_cb)
    ((Ev *)ctx)->display_in_xnav_cb( ((Ev *)ctx)->parent_ctx, arp);
}

void Ev::ala_display_in_xnav_cb( void *ctx, pwr_tAttrRef *arp)
{
  if ( ((Ev *)ctx)->display_in_xnav_cb)
    ((Ev *)ctx)->display_in_xnav_cb( ((Ev *)ctx)->parent_ctx, arp);
}

void Ev::blk_display_in_xnav_cb( void *ctx, pwr_tAttrRef *arp)
{
  if ( ((Ev *)ctx)->display_in_xnav_cb)
    ((Ev *)ctx)->display_in_xnav_cb( ((Ev *)ctx)->parent_ctx, arp);
}


void Ev::ala_help_cb( void *ctx, const char *key)
{
  if ( ((Ev *)ctx)->help_cb)
    ((Ev *)ctx)->help_cb( ((Ev *)ctx)->parent_ctx, key);
}

int Ev::ala_is_authorized_cb( void *ctx, unsigned int access)
{
  if ( ((Ev *)ctx)->is_authorized_cb)
    return ((Ev *)ctx)->is_authorized_cb( ((Ev *)ctx)->parent_ctx, access);
  return 0;
}

int Ev::sala_acknowledge_cb( void *ctx, mh_sEventId *id)
{
  Ev *ev = (Ev *)ctx;

  mh_sEventId lid = *id;
  ev->ala->ack( id);
  ev->eve->ack( id);
  for ( int i = 0; i < ev->sala_cnt; i++)
    ev->sala[i]->ack( id);
  mh_OutunitAck( &lid);
  return 1;
}

void Ev::sala_copy_list_cb( void *ctx, EvList *evl)
{
  Ev *ev = (Ev *)ctx;

  ev->ala->copy_list( evl);
}

void Ev::sala_close_cb( void *ctx, EvAla *sala)
{
  Ev *ev = (Ev *)ctx;

  bool found = false;
  for ( int i = 0; i < ev->sala_cnt; i++) {
    if ( ev->sala[i] == sala)
      found = true;
    if ( found && i != ev->sala_cnt - 1)
      ev->sala[i] = ev->sala[i+1];
  }
  if ( found)
    ev->sala_cnt--;
}

void Ev::eve_activate_print()
{
  char title[80];

  strcpy( title, Lng::translate( "Event List"));
  eve->print( title);
}

void Ev::ala_activate_print()
{
  char title[80];

  strcpy( title, Lng::translate( "Alarm List"));
  ala->print( title);
}

void Ev::blk_activate_print()
{
  char title[80];

  strcpy( title, Lng::translate( "Blocked Alarms"));
  blk->print( title);
}

void Ev::eve_activate_ack_last()
{
  mh_sEventId *id;
  int sts;

  if (is_authorized_cb && !is_authorized_cb( parent_ctx, pwr_mAccess_RtEventsAck | pwr_mAccess_System))
    return;

  sts = ala->get_last_not_acked( &id);
  if ( EVEN(sts)) return;

  mh_sEventId lid = *id;
  ala->ack( id);
  eve->ack( id);
  for ( int i = 0; i < sala_cnt; i++)
    sala[i]->ack( id);
  mh_OutunitAck( &lid);
}

void Ev::ala_activate_ack_last()
{
  mh_sEventId *id;
  int sts;

  if (is_authorized_cb && !is_authorized_cb( parent_ctx, pwr_mAccess_RtEventsAck | pwr_mAccess_System))
    return;

  if ( ala->brow == ala->browbase) {
    // Flat view, acknowledge last
    sts = ala->get_last_not_acked( &id);
    if ( EVEN(sts)) return;
  }
  else {
    // Tree view, acknowledge selected
    ItemAlarm *item;
    pwr_tAName eventname;

    sts = ala->get_selected_event( eventname, &item);
    if ( EVEN(sts)) return;

    switch ( item->type) {
    case evlist_eItemType_Alarm:
      id = &item->eventid;
      break;
    default:
      return;
    }
  }

  mh_sEventId lid = *id;
  ala->ack( id);
  eve->ack( id);
  for ( int i = 0; i < sala_cnt; i++)
    sala[i]->ack( id);
  mh_OutunitAck( &lid);
}

void Ev::eve_activate_ack_all()
{
  if (is_authorized_cb && !is_authorized_cb( parent_ctx, pwr_mAccess_RtEventsAck | pwr_mAccess_System))
    return;

  ack_all();
}

void Ev::eve_activate_help()
{
  if ( help_cb)
    (help_cb)( parent_ctx, "opg_eventlist");
}

void Ev::eve_activate_helpevent()
{
  char	eventname[80];
  int 	sts;
  ItemAlarm *item;

  if ( help_cb) {
    sts = ev->eve->get_selected_event( eventname, &item);
    if( ODD(sts)) {
      wow->DisplayText( eventname, item->eventmoretext);
    }
  }
}

void Ev::ala_activate_help()
{
  if ( help_cb)
    (help_cb)( parent_ctx, "opg_alarmlist");
}

void Ev::ala_activate_helpevent()
{
  char	eventname[80];
  int 	sts;
  ItemAlarm *item;

  if ( help_cb) {
    sts = ala->get_selected_event( eventname, &item);
    if( ODD(sts)) {
      wow->DisplayText( eventname, item->eventmoretext);
    }
  }
}

void Ev::blk_activate_help()
{
  if ( help_cb)
    (help_cb)( parent_ctx, "opg_blocklist");
}

int	Ev::get_alarm_info( evlist_sAlarmInfo *info)
{
  return ala->get_alarm_info( info);
}

int	Ev::outunit_connect( pwr_tObjid	user)
{
  int		sts;

  // Wait for mh has flagged initizated
  mh_UtilWaitForMh();

  sts = mh_OutunitConnect(
		user,
		mh_eOutunitType_Operator,
		0,
		mh_ack_bc,
		mh_alarm_bc,
		mh_block_bc,
		mh_cancel_bc,
		mh_clear_alarmlist_bc,
		mh_clear_blocklist_bc,
		mh_info_bc,
		mh_return_bc
		);
  if (EVEN(sts)) return sts;

  connected = 1;
  return XNAV__SUCCESS;
}

void Ev::update( double scantime)
{
  int sts;
  int nodraw_set = 0;

  sts = mh_OutunitReceive();     
  while (ODD(sts))
  {
    if ( !nodraw_set)
    {
      eve->set_nodraw();
      ala->set_nodraw();
      nodraw_set = 1;
    }
    sts = mh_OutunitReceive();     
  }
  if ( nodraw_set)
  {
    eve->reset_nodraw();
    ala->reset_nodraw();
  }

  ala->flash();
  for ( int i = 0; i < sala_cnt; i++)
    sala[i]->update();
  if ( beep)
    ala->beep( scantime);
}      

void Ev::ack_last_prio( unsigned long type, unsigned long prio)
{
  mh_sEventId 	*id;
  int		sts;

  if (is_authorized_cb && !is_authorized_cb( parent_ctx, pwr_mAccess_RtEventsAck | pwr_mAccess_System))
    return;

  sts = ala->get_last_not_acked_prio( &id, type, prio);
  if ( ODD(sts))
  {
    mh_sEventId lid = *id;

    ala->ack( id);
    eve->ack( id);
    for ( int i = 0; i < sala_cnt; i++)
      sala[i]->ack( id);
    mh_OutunitAck( &lid);
  }
}

void Ev::ack_all()
{
  mh_sEventId *id;
  int sts;

  if (is_authorized_cb && !is_authorized_cb( parent_ctx, pwr_mAccess_RtEventsAck | pwr_mAccess_System))
    return;

  sts = ala->get_last_not_acked( &id);
  while ( ODD(sts)) {
    mh_sEventId lid = *id;
    ala->ack( id);
    eve->ack( id);
    for ( int i = 0; i < sala_cnt; i++)
      sala[i]->ack( id);
    mh_OutunitAck( &lid);

    sts = ala->get_last_not_acked( &id);
  }
}

int Ev::get_last_not_acked_prio( mh_sEventId **id, unsigned long type, 
	unsigned long prio)
{
  return ala->get_last_not_acked_prio( id, type, prio);
}

void Ev::create_aliaslist( void *up)
{
  char alias_array[2][80];
  int nr;
  int i, j;
  int alias_size;
  ev_sAlias dum;
  pwr_sClass_OpPlace *userp = (pwr_sClass_OpPlace *)up;
  int listsize = MIN( sizeof(userp->EventSelectList)/sizeof(userp->EventSelectList[0]),
		      sizeof(alias_list)/sizeof(alias_list[0]));
						    
  for ( i = 0, j = 0; i < listsize; i++) {
    strcpy( alias_list[i].Alias, "");
    strcpy( alias_list[i].Object, "");
  }
  for ( i = 0, j = 0; i < listsize; i++) {
    nr = dcli_parse( userp->EventSelectList[i], " 	", "",
	     (char *) alias_array, sizeof( alias_array)/sizeof( alias_array[0]), 
	     sizeof( alias_array[0]), 0);
    if ( nr < 2)
      continue;

    cdh_ToUpper( alias_list[j].Object, alias_array[0]);
    strncpy( alias_list[j].Alias, alias_array[1], sizeof(alias_list[j].Alias));
    alias_list[j].Alias[sizeof(alias_list[j].Alias)-1] = 0;
    j++;
  }
  alias_size = j;

  // Order
  for ( i = alias_size - 1; i > 0; i--) {
    for ( j = 0; j < i; j++) {
      if ( strcmp( alias_list[j].Object, alias_list[j+1].Object) <= 0) {
	memcpy( &dum, &alias_list[j+1], sizeof(dum));
	memcpy( &alias_list[j+1], &alias_list[j], sizeof(dum));
	memcpy( &alias_list[j], &dum, sizeof(dum));
      }
    }
  }
}

char *Ev::name_to_alias( char *name)
{
  char oname[120];
  static char alias[40];

  cdh_ToUpper( oname, name);

  for ( int i = 0; i < (int)(sizeof(alias_list)/sizeof(alias_list[0])); i++) {
    if ( alias_list[i].Alias[0] == 0)
      break;
    if ( strcmp( alias_list[i].Object, "") != 0 && 
	 strncmp( oname, alias_list[i].Object, strlen(alias_list[i].Object)) == 0) {
      strcpy( alias, alias_list[i].Alias);
      return alias;
    }	 
  }
  strcpy( alias, "");
  return alias;
}

pwr_tStatus Ev::mh_ack_bc( mh_sAck *MsgP)
{
  if ( ev->eve_display_ack)
  {
    // Insert in eve
    ev->eve->event_ack( MsgP);
  }
  ev->ala->event_ack( MsgP);
  for ( int i = 0; i < ev->sala_cnt; i++)
    ev->sala[i]->mh_ack( MsgP);

  if ( ev->update_info_cb)
    ev->update_info_cb( ev->parent_ctx);
  return 1;
}

pwr_tStatus Ev::mh_return_bc( mh_sReturn *MsgP)
{
  if ( ev->eve_display_return)
  {
    // Insert in eve
    ev->eve->event_return( MsgP);
  }
  ev->ala->event_return( MsgP);
  for ( int i = 0; i < ev->sala_cnt; i++)
    ev->sala[i]->mh_return( MsgP);

  if ( ev->update_info_cb)
    ev->update_info_cb( ev->parent_ctx);
  return 1;
}

pwr_tStatus Ev::mh_alarm_bc( mh_sMessage *MsgP)
{
  ev->eve->event_alarm( MsgP);
  ev->ala->event_alarm( MsgP);
  for ( int i = 0; i < ev->sala_cnt; i++)
    ev->sala[i]->mh_alarm( MsgP);
  if ( ev->update_info_cb)
    ev->update_info_cb( ev->parent_ctx);
  if ( ev->pop_cb) {
    int pop = 0;

    switch ( ((mh_sMsgInfo *)MsgP)->EventPrio) {
    case mh_eEventPrio_A:
      if ( ev->pop_mask & pwr_mOpWindPopMask_Aalarm)
	pop = 1;
      break;
    case mh_eEventPrio_B:
      if ( ev->pop_mask & pwr_mOpWindPopMask_Balarm)
	pop = 1;
      break;
    case mh_eEventPrio_C:
      if ( ev->pop_mask & pwr_mOpWindPopMask_Calarm)
	pop = 1;
      break;
    case mh_eEventPrio_D:
      if ( ev->pop_mask & pwr_mOpWindPopMask_Dalarm)
	pop = 1;
      break;
    default: ;
    }
    if ( pop)
      ev->pop_cb( ev->parent_ctx);
  }
  return 1;
}

pwr_tStatus Ev::mh_block_bc( mh_sBlock *MsgP)
{
  ev->blk->event_block( MsgP);
  ev->eve->event_block( MsgP);
  if ( ev->update_info_cb)
    ev->update_info_cb( ev->parent_ctx);
  return 1;
}

pwr_tStatus Ev::mh_cancel_bc( mh_sReturn *MsgP)
{
  ev->ala->event_cancel( MsgP);
  for ( int i = 0; i < ev->sala_cnt; i++)
    ev->sala[i]->mh_cancel( MsgP);

  if ( ev->update_info_cb)
    ev->update_info_cb( ev->parent_ctx);
  return 1;
}

pwr_tStatus Ev::mh_info_bc( mh_sMessage *MsgP)
{
  ev->eve->event_info( MsgP);
  ev->ala->event_info( MsgP);
  for ( int i = 0; i < ev->sala_cnt; i++)
    ev->sala[i]->mh_info( MsgP);
  if ( ev->update_info_cb)
    ev->update_info_cb( ev->parent_ctx);
  if ( ev->pop_mask & pwr_mOpWindPopMask_InfoMsg)
    ev->pop_cb( ev->parent_ctx);

  return 1;
}

pwr_tStatus Ev::mh_clear_alarmlist_bc( pwr_tNodeIndex nix)
{
  ev->ala->event_clear_alarmlist( nix);
  for ( int i = 0; i < ev->sala_cnt; i++)
    ev->sala[i]->mh_clear_alarmlist( nix);
  return 1;
}

pwr_tStatus Ev::mh_clear_blocklist_bc( pwr_tNodeIndex nix)
{
  ev->blk->event_clear_alarmlist( nix);
  return 1;
}

pwr_tStatus Ev::set_view(pwr_tOid view)
{
  pwr_tStatus sts;

  sts = ala->set_view( view);
  if ( ODD(sts)) {
    pwr_tString80 name;

    if ( cdh_ObjidIsNull( view)) {
      strcpy( name, "Alarm List");
    }
    else {
      pwr_tAttrRef name_ar, ar;

      ar = cdh_ObjidToAref( view);
      sts = gdh_ArefANameToAref( &ar, "Name", &name_ar);
      if (EVEN(sts)) return sts;

      sts = gdh_GetObjectInfoAttrref( &name_ar, name, sizeof(name));
      if (EVEN(sts)) return sts;
    }
    set_title_ala( name);
  }
  return sts;
}

void Ev::view_shift() 
{
  pwr_sClass_OpPlace *opp;
  pwr_tStatus sts;

  sts = gdh_ObjidToPointer( user, (pwr_tAddress *) &opp);
  if ( EVEN(sts)) return;

  if ( cdh_ObjidIsNull(ala->current_view)) {
    set_view( opp->AlarmViews[0]);
  }
  else {
    for ( unsigned int i = 0; i < sizeof(opp->AlarmViews)/sizeof(opp->AlarmViews[0]); i++) {
      if ( cdh_ObjidIsEqual( ala->current_view, opp->AlarmViews[i])) {
	if ( i == sizeof(opp->AlarmViews)/sizeof(opp->AlarmViews[0]) - 1)
	  set_view( pwr_cNObjid);
	else
	  set_view( opp->AlarmViews[i+1]);
	break;
      }	   
    }
  }
}
