/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2012 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */

#include "glow_std.h"

#include <stdio.h>
#include <stdlib.h>

#include "glow.h"
#include "xtt_xnav.h"
#include "rt_gdh.h"
#include "rt_gdh_msg.h"
#include "co_cdh.h"
#include "co_dcli.h"
#include "co_time.h"
#include "glow_growctx.h"
#include "glow_growapi.h"
#include "co_lng.h"
#include "xtt_ge.h"
#include "ge_graph.h"
#include "xtt_log.h"

void XttGe::eventlog_enable( int enable) 
{ 
  Graph::eventlog_enable( enable);
}

void XttGe::graph_init_cb( void *client_data)
{
  XttGe *ge = (XttGe *) client_data;
  char fname[120];
  int		default_width;
  int		default_height;
  int		sts;

  strncpy( fname, ge->filename, sizeof(fname));
  if ( ! strrchr( fname, '.'))
    strcat( fname, ".pwg");
  ge->graph->open( fname);

  ge->graph->init_trace();

  if ( ge->width == 0 || ge->height == 0) {
    sts = ge->graph->get_default_size( &default_width, &default_height);
    if ( ODD(sts)) {
      ge->set_size( default_width, default_height);
    }
  }

  ge->graph->set_default_layout();

}

int XttGe::graph_close_cb( void *client_data)
{
  XttGe *ge = (XttGe *) client_data;

  if ( ge->options & ge_mOptions_Embedded)
    return 0;

  delete ge;
  return 1;
}

int XttGe::ge_command_cb( void *ge_ctx, char *cmd)
{
  XttGe	*ge = (XttGe *)ge_ctx;
  int		sts;

  if ( ge->command_cb)
  {
    sts = (ge->command_cb)( ge->parent_ctx, cmd, ge_ctx);
    return sts;
  }
  return 0;
}

int XttGe::ge_sound_cb( void *ge_ctx, pwr_tAttrRef *aref)
{
  XttGe	*ge = (XttGe *)ge_ctx;

  if ( ge->sound_cb)
    return (ge->sound_cb)( ge->parent_ctx, aref);

  return 0;
}

void XttGe::ge_display_in_xnav_cb( void *ge_ctx, pwr_sAttrRef *arp)
{
  XttGe	*ge = (XttGe *)ge_ctx;

  if ( ge->display_in_xnav_cb)
    (ge->display_in_xnav_cb)( ge->parent_ctx, arp);
}

void XttGe::ge_popup_menu_cb( void *ge_ctx, pwr_sAttrRef attrref,
			   unsigned long item_type, unsigned long utility, 
			   char *arg, int x, int y)
{
  XttGe	*ge = (XttGe *)ge_ctx;

  if ( ge->popup_menu_cb)
    (ge->popup_menu_cb)( ge->parent_ctx, attrref, item_type, utility, 
			    arg, x, y);
}

int XttGe::ge_call_method_cb( void *ge_ctx, char *method, char *filter,
			   pwr_sAttrRef attrref, unsigned long item_type, 
			   unsigned long utility, char *arg)
{
  XttGe	*ge = (XttGe *)ge_ctx;

  if ( ge->call_method_cb)
    return (ge->call_method_cb)( ge->parent_ctx, method, filter, attrref, item_type, utility, 
			    arg);
  else return 0;
}

int XttGe::ge_is_authorized_cb( void *ge_ctx, unsigned int access)
{
  XttGe	*ge = (XttGe *)ge_ctx;

  if ( ge->is_authorized_cb)
    return (ge->is_authorized_cb)( ge->parent_ctx, access);
  return 0;
}

int XttGe::ge_get_current_objects_cb( void *ge_ctx, pwr_sAttrRef **alist,
				   int **is_alist)
{
  XttGe	*ge = (XttGe *)ge_ctx;

  if ( ge->get_current_objects_cb)
    return (ge->get_current_objects_cb)( ge->parent_ctx, alist, is_alist);
  return 0;
}

void XttGe::ge_eventlog_cb( void *ge_ctx, void *data, unsigned int size)
{
  XttGe	*ge = (XttGe *)ge_ctx;

  if ( ge->eventlog_cb)
    (ge->eventlog_cb)( ge->parent_ctx, ge, xttlog_eCategory_Event, data, size);
}

void XttGe::message_cb( void *ctx, char severity, const char *msg)
{
  ((XttGe *)ctx)->message( severity, msg);
}

void XttGe::message( char severity, const char *msg)
{
  if ( strcmp( msg, "") != 0)
    printf("** XttGe: %s\n", msg);
}


int XttGe::set_object_focus( const char *name, int empty)
{
  return graph->set_object_focus( name, empty);
}

int XttGe::set_folder_index( const char *name, int idx)
{
  return graph->set_folder_index( name, idx);
}

int XttGe::set_subwindow_source( const char *name, char *source, char *object)
{
  return graph->set_subwindow_source( name, source, object);
}

XttGe::~XttGe()
{
}

void XttGe::print()
{
  pwr_tFileName filename;
  pwr_tCmd cmd;

  dcli_translate_filename( filename, "$pwrp_tmp/graph.ps");
  graph->print( filename);

  sprintf( cmd, "$pwr_exe/rt_print.sh %s 1", filename);
  system(cmd);
}

void XttGe::export_image( char *filename)
{
  pwr_tFileName fname;

  dcli_translate_filename( fname, filename);
  graph->export_image( fname);
}

void XttGe::swap( int mode)
{
  graph->swap( mode);
}

void XttGe::event_exec( int type, void *event, unsigned int size) 
{ 
  switch ( type) {
  case xttlog_eCategory_Event:
    graph->event_exec( event, size);
    break;
  case xttlog_eCategory_GeConfirmOk:
    confirm_reply( 1);
    break;
  case xttlog_eCategory_GeConfirmCancel:
    confirm_reply( 0);
    break;
  default: ;
  }
}

XttGe::XttGe( void *xg_parent_ctx, const char *xg_name, const char *xg_filename,
	      int xg_scrollbar, int xg_menu, int xg_navigator, int xg_width, int xg_height,
	      int x, int y, double scan_time, const char *object_name,
	      int use_default_access, unsigned int access, unsigned int xg_options,
	      int (*xg_command_cb) (void *, char *, void *),
	      int (*xg_get_current_objects_cb) (void *, pwr_sAttrRef **, int **),
	      int (*xg_is_authorized_cb) (void *, unsigned int)) :
  parent_ctx(xg_parent_ctx), scrollbar(xg_scrollbar),
  navigator(xg_navigator), menu(xg_menu), current_value_object(0),
  current_confirm_object(0), value_input_open(0), confirm_open(0), 
  command_cb(xg_command_cb), close_cb(0), help_cb(0), display_in_xnav_cb(0), 
  is_authorized_cb(xg_is_authorized_cb), popup_menu_cb(0), call_method_cb(0), 
  get_current_objects_cb(xg_get_current_objects_cb), sound_cb(0), eventlog_cb(0),
  width(xg_width), height(xg_height), options(xg_options)
{
  strcpy( filename, xg_filename);
  strcpy( name, xg_name);
}

