/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2012 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */

#ifndef xtt_ge_h
#define xtt_ge_h

#ifndef pwr_h
# include "pwr.h"
#endif
#include "glow.h"

class Graph;

class XttGe {
 public:
  void 		*parent_ctx;
  pwr_tAName   	name;
  Graph		*graph;
  pwr_tFileName	filename;
  int		scrollbar;
  int		navigator;
  int		menu;
  void		*current_value_object;
  void		*current_confirm_object;
  int		value_input_open;
  int		confirm_open;
  int		(*command_cb)(void *, char *, void *);
  void		(*close_cb)(void *, void *);
  void		(*help_cb)(void *, const char *key);
  void		(*display_in_xnav_cb)(void *, pwr_sAttrRef *);
  int		(*is_authorized_cb)(void *, unsigned int);
  void          (*popup_menu_cb)(void *, pwr_sAttrRef, unsigned long,
					 unsigned long, char *, int x, int y); 
  int         	(*call_method_cb)(void *, const char *, const char *, pwr_sAttrRef,
				  unsigned long, unsigned long, char *);
  int           (*get_current_objects_cb)(void *, pwr_sAttrRef **, int **);
  int           (*sound_cb)(void *, pwr_tAttrRef *);
  void          (*eventlog_cb)(void *, void *, int, void *, unsigned int);
  int		width;
  int		height;
  unsigned int	options;

  XttGe( void *parent_ctx, const char *name, const char *filename,
	 int scrollbar, int menu, int navigator, int width, int height,
	 int x, int y, double scan_time, const char *object_name, int use_default_access,
	 unsigned int access, unsigned int options,
	 int (*xg_command_cb) (void *, char *, void *),
	 int (*xg_get_current_objects_cb) (void *, pwr_sAttrRef **, int **),
	 int (*xg_is_authorized_cb) (void *, unsigned int));
  virtual ~XttGe();

  virtual void pop() {}
  virtual void set_size( int width, int height) {}
  virtual void confirm_reply( int ok) {}
  virtual void *get_widget() { return 0;}

  void message( char severity, const char *msg);
  void print();
  void export_image( char *filename);
  int set_object_focus( const char *name, int empty);
  int set_folder_index( const char *name, int idx);
  int set_subwindow_source( const char *name, char *source, char *object);
  void swap( int mode);
  void event_exec( int type, void *event, unsigned int size);

  static void graph_init_cb( void *client_data);
  static int graph_close_cb( void *client_data);
  static int ge_command_cb( void *ge_ctx, char *command);
  static int ge_sound_cb( void *ge_ctx, pwr_tAttrRef *aref);
  static void ge_display_in_xnav_cb( void *ge_ctx, pwr_sAttrRef *arp);
  static void ge_popup_menu_cb( void *ge_ctx, pwr_sAttrRef attrref,
			     unsigned long item_type, unsigned long utility, 
			     char *arg, int x, int y);
  static int ge_call_method_cb( void *ge_ctx, char *method, char *filter,
			     pwr_sAttrRef attrref, unsigned long item_type, 
			     unsigned long utility, char *arg);
  static int ge_is_authorized_cb( void *ge_ctx, unsigned int access);
  static int ge_get_current_objects_cb( void *ge_ctx, pwr_sAttrRef **alist,
				     int **is_alist);
  static void ge_eventlog_cb( void *ge_ctx, void *value, unsigned int size);
  static void message_cb( void *ctx, char severity, const char *msg);
  static void eventlog_enable( int enable);
};

#endif






