/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2012 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 */

/* xtt_localdb.h -- Local parameter database */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

extern "C" {
#include "pwr.h"
}

#include "xtt_localdb.h"
#include "rt_xnav_msg.h"

int	XNavLdb::add( char *name, xnav_eLdbType type, void *value_p)
{
  xnav_sLdbEntry 	*entry_p;
  xnav_eLdbType		get_type;
  void			*get_value_p;

  // Check that its not inserted
  if ( ODD( get( name, &get_type, &get_value_p)))
    return XNAV__ENTRYALREADYEXIST;

  entry_p = (xnav_sLdbEntry *) malloc( sizeof(*entry_p));
  strcpy( entry_p->name, name);
  entry_p->type = type;
  entry_p->value_p = value_p;
  entry_p->next = root;
  root = entry_p;

  return 1;
}

int	XNavLdb::remove( char *name)
{
  xnav_sLdbEntry *entry_p = root;
  xnav_sLdbEntry *prev_p = NULL;

  while ( entry_p)
  {
    if ( strcmp( entry_p->name, name) == 0)
    {
      if ( entry_p == NULL)
        root = entry_p->next;
      else
        prev_p->next = entry_p->next;
      free( entry_p);
      return 1;
    }
    entry_p = entry_p->next;
    prev_p = entry_p;
  }
  return 0;
}

void	XNavLdb::remove_all()
{
  xnav_sLdbEntry *entry_p = root;
  xnav_sLdbEntry *tmp_p;

  while ( entry_p)
  {
    tmp_p = entry_p->next;
    free( entry_p);
    entry_p = tmp_p;
  }
}

int	XNavLdb::get( char *name, xnav_eLdbType *type, void **value_p)
{
  xnav_sLdbEntry *entry_p = root;

  while ( entry_p)
  {
    if ( strcmp( entry_p->name, name) == 0)
    {
      *type = entry_p->type;
      *value_p = entry_p->value_p;
      return 1;
    }
    entry_p = entry_p->next;
  }
  return 0;
}
